/*******************************************************************************
 * Copyright 2018 Frank Mnster
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler;

/**
 * Interface fr alle QrCodeHandlers
 *
 * @author Frank Mnster
 */
public interface QrCodeHandler {

    /**
     * Wird aufgerufen, um zu berprfen, ob der Handler den QR-Code verarbeiten kann.
     *
     * @param qrCode
     *            - Zu bearbeitender QR-Code
     * @return <b>true</b>, falls der Handler den QR-Code bearbeiten kann, <b>false</b> sonst
     */
    boolean canHandle(String qrCode);

    /**
     * Zeigt an, ob der Handler fr berweisungen genutzt wird. Dies wird bei der Verarbeitung von PDF Dateien bentigt.
     *
     * @return <b>true</b>, falls der Handler berweisungen bearbeitet, <b>false</b> sonst
     */
    boolean isUeberweisung();

    /**
     * Liefert der Namen des Handlers zurck. Der Name ist frei whlbar, sollte aber aussagekrftig sein
     *
     * @return <String> - Name des Handlers
     */
    String getHandlerName();

    /**
     * Wird aufgerufen, um einen QR-Code zu bearbeiten
     *
     * @param qrCode
     *            Der QR-Code als <b>String</b>
     * @throws QrCodeHandlerException
     *             Falls der QR-Code nicht bearbeitet werden kann. Vorher mittels <b>canHandle()</b> berprfen
     */
    void handleQrCode(String qrCode) throws QrCodeHandlerException;
}
