/*******************************************************************************
 * Copyright 2018 Frank Mnster
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner;

import de.frankmuenster.jameica.qrscanner.cliurlhandler.CliUrlHandler;
import de.frankmuenster.jameica.qrscanner.gui.navigation.QrCodeScannerExtension;
import de.willuhn.jameica.gui.extension.ExtensionRegistry;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.StartupParams;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Haupt-Klasse des jameica.qrscanner Plug-ins.
 *
 * @author Frank Mnster
 */
public class QrCodeScanner extends AbstractPlugin {

    /**
     * Standardkonsturktor
     */
    public QrCodeScanner() {
        super();
    }

    /**
     * Methode die beim Starten des Plug-Ins geladen wird
     *
     * @see de.willuhn.jameica.plugin.AbstractPlugin#init()
     */
    @Override
    public void init() throws ApplicationException {
        super.init();
        this.ladeHandler();
        WebcamWatchdog.getInstance();
        // Menu des Jameica-'Plugins' erweitern
        ExtensionRegistry.register(new QrCodeScannerExtension(), "jameica.start"); //$NON-NLS-1$
        // Check ob mittels bank:// URL gestartet
        final StartupParams startParams = Application.getStartupParams();
        final String[] params = startParams.getParams();
        for (final String string : params) {
            if (string.startsWith("bank://")) { //$NON-NLS-1$
                Logger.debug("Hibiscus mittels BezahlCode-URL handler gestartet. UrlHandler anlegen..."); //$NON-NLS-1$
                Application.getMessagingFactory().registerMessageConsumer(new CliUrlHandler(string)); // NOPMD
            }
        }
    }

    /**
     * Hier werden alle QR-Code Handler einmal geladen, damit sie in sich in der Registry anmelden.
     */
    private void ladeHandler() {
        // Laden der verschiedenen code handler
        this.touchClass("de.frankmuenster.jameica.qrcodehandler.bezahlcode.BezahlcodeSinglePaymentSepaHandler"); //$NON-NLS-1$
        this.touchClass("de.frankmuenster.jameica.qrcodehandler.bezahlcode.BezahlcodeSinglePaymentHandler"); //$NON-NLS-1$
        this.touchClass("de.frankmuenster.jameica.qrcodehandler.bezahlcode.BezahlcodeContactV2Handler"); //$NON-NLS-1$
        this.touchClass("de.frankmuenster.jameica.qrcodehandler.bcdsepa.BcdSepa_001_SCT"); //$NON-NLS-1$
        this.touchClass("de.frankmuenster.jameica.qrcodehandler.bcdsepa.BcdSepa_002_SCT"); //$NON-NLS-1$
    }

    /**
     * Methode die fr einen angegebenen Klassenname ein Class.forName durchfhrt.
     *
     * @param className
     *            Name der Klasse die geladen werden soll.
     */
    private void touchClass(final String className) {
        if (className != null && !className.isEmpty()) {
            try {
                Class.forName(className);
            } catch (final ClassNotFoundException e) {
                Logger.debug("QrCodeHandler-Klasse " + className + " nicht gefunden."); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
    }

    /**
     * @see de.willuhn.jameica.plugin.AbstractPlugin#shutDown()
     */
    @Override
    public void shutDown() {
        Logger.debug("Shuting down Plugin QrCodeScanner."); //$NON-NLS-1$
        super.shutDown();
    }
}
