/*******************************************************************************
 * Copyright 2018 Frank Mnster
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.utils;

import java.util.HashSet;
import java.util.Set;

/**
 * ContryCode Klasse zum validiern, ob IBAN udn BIC zu EUCountry gehoeren. Aus Forumeintrag
 * http://www.tutorials.de/java/259520-iban-bic.html
 *
 * @see <a href="www.tutorials.de/java/259520-iban-bic.html">http://www.tutorials.de/java/259520-iban-bic.html</a>
 * @author takidoso / Frank Mnster
 */
public final class EUCountryCodes {

    /**
     * Liste der Lnder fr einen gltiger IBAN Code
     */
    static private Set<String> ibanCountryCodes = new HashSet<>();

    /**
     * Liste der Lnder fr einen gltiger BIC Code
     */
    static private Set<String> bicCountryCodes = new HashSet<>();
    static {
        EUCountryCodes.ibanCountryCodes.add("AT"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("BE"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("CY"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("CZ"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("DE"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("DK"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("EE"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("ES"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("FI"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("FR"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("GB"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("GI"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("GR"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("HU"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("IE"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("IS"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("IT"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("LI"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("LT"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("LU"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("LV"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("MT"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("NL"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("NO"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("PL"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("PT"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("SE"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("SI"); //$NON-NLS-1$
        EUCountryCodes.ibanCountryCodes.add("SK"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("AT"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("BE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("CY"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("CZ"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("DE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("DK"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("EE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("ES"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("FI"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("FR"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("GB"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("IE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("GF"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("GI"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("GP"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("GR"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("HU"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("IE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("IS"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("IT"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("LI"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("LT"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("LU"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("LV"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("MQ"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("MT"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("NL"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("NO"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("PL"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("PT"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("RE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("SE"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("SI"); //$NON-NLS-1$
        EUCountryCodes.bicCountryCodes.add("SK"); //$NON-NLS-1$
    }

    private EUCountryCodes() {
    }

    /**
     * Ueberprueft Eingabestring (im BIC-Format) auf Laenderkennung an Stelle 5 und 6.
     *
     * @param bic
     *            <Sting> zu ueberpruefende BIC
     * @return <b>true</b>, wenn EU Laenderkennung gefunden, <b>false</b> sonst.
     */
    static public boolean isBicEUCountry(final String bic) {
        return EUCountryCodes.bicCountryCodes.contains(bic.substring(4, 6));
    }

    /**
     * Ueberprueft Eingabestring (im IBAN-Format) auf Laenderkennung an Stelle 1 und 2.
     *
     * @param iban
     *            <Sting> zu berprfende IBAN
     * @return <b>true</b>, wenn EU Lnderkennung gefunden, <b>false</b> sonst.
     */
    static public boolean isIbanEUCountry(final String iban) {
        return EUCountryCodes.ibanCountryCodes.contains(iban.substring(0, 2));
    }
}
